@@ -7,7 +7,6 @@
 
 #pragma once
 
-#define HANDY_USE_UNSTABLE_API
 #include <handy.h>
 
 #include "users/chatty-item.h"
@@ -15,7 +14,7 @@
 G_BEGIN_DECLS
 
 #define CHATTY_TYPE_NEW_CHAT_DIALOG (chatty_new_chat_dialog_get_type())
-G_DECLARE_FINAL_TYPE (ChattyNewChatDialog, chatty_new_chat_dialog, CHATTY, NEW_CHAT_DIALOG, HdyDialog)
+G_DECLARE_FINAL_TYPE (ChattyNewChatDialog, chatty_new_chat_dialog, CHATTY, NEW_CHAT_DIALOG, GtkDialog)
 
 
 GtkWidget *chatty_new_chat_dialog_new (GtkWindow *parent_window);
