@@ -7,13 +7,12 @@
 
 #pragma once
 
-#define HANDY_USE_UNSTABLE_API
 #include <handy.h>
 
 G_BEGIN_DECLS
 
 #define CHATTY_TYPE_NEW_MUC_DIALOG (chatty_new_muc_dialog_get_type())
-G_DECLARE_FINAL_TYPE (ChattyNewMucDialog, chatty_new_muc_dialog, CHATTY, NEW_MUC_DIALOG, HdyDialog)
+G_DECLARE_FINAL_TYPE (ChattyNewMucDialog, chatty_new_muc_dialog, CHATTY, NEW_MUC_DIALOG, GtkDialog)
 
 
 GtkWidget *chatty_new_muc_dialog_new (GtkWindow *parent_window);
